/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



#include "boardlines.h"
#include "point.h"

#include <math.h>
#include <assert.h>

double min(double a, double b) { return (a < b) ? a : b; }

BoardLines::BoardLines(istream &in) {
  size = 0;
  dat = NULL;
  lines = NULL;
  
  // first read the 4 corners
  in.read(&corners[0][0], 2);
  in.read(&corners[1][0], 2);
  
  in.read(&corners[0][1], 2);
  in.read(&corners[1][1], 2);

  in.read(&corners[0][2], 2);
  in.read(&corners[1][2], 2);

  in.read(&corners[0][3], 2);
  in.read(&corners[1][3], 2);
  
  // now read the color
  lineColor = Color(in);
}

BoardLines::~BoardLines() {
  if (lines != NULL) delete lines;
  if (dat != NULL) delete [] dat;
}

void BoardLines::setBoardSize(Uint8 sz) {
  if (dat != 0) {
    delete [] dat;
    dat = NULL;
  }
  
  if (lines != NULL) { 
    delete lines;
    lines = NULL;
  }
  
  size = sz;
  dat = new Sint16[sz*sz*3];
  
  double *vl = new double[sz*3];
  double *hl = new double[sz*3];
  
  double xs, ys, xe, ye, d, x, y;
  
  for (int l = 0; l < sz; l++) {
  
    // calculate the start the endpoint of the vertical lines
    xs = corners[0][0] + (corners[0][1] - corners[0][0]) * (l + 0.5) / sz;
    ys = corners[1][0] + (corners[1][1] - corners[1][0]) * (l + 0.5) / sz;
    
    xe = corners[0][3] + (corners[0][2] - corners[0][3]) * (l + 0.5) / sz;
    ye = corners[1][3] + (corners[1][2] - corners[1][3]) * (l + 0.5) / sz;
    
    // calculate the parameter of these lines
    vl[3*l] = ys-ye;
    vl[3*l+1] = xe-xs;
    vl[3*l+2] = (vl[3*l]*xs + vl[3*l+1]*ys) * -1;
    
    // not the same for horizontal lines
    
    xs = corners[0][1] + (corners[0][2] - corners[0][1]) * (l + 0.5) / sz;
    ys = corners[1][1] + (corners[1][2] - corners[1][1]) * (l + 0.5) / sz;
    
    xe = corners[0][0] + (corners[0][3] - corners[0][0]) * (l + 0.5) / sz;
    ye = corners[1][0] + (corners[1][3] - corners[1][0]) * (l + 0.5) / sz;
    
    // calculate the parameter of these lines
    
    hl[3*l] = ys-ye;
    hl[3*l+1] = xe-xs;
    hl[3*l+2] = (hl[3*l]*xs + hl[3*l+1]*ys) * -1;
  }
  
  // now calculate the cutting points between each pair of lines
  for (int h = 0; h < sz; h++)
    for (int v = 0; v < sz; v++) {
      d = vl[3*v]*hl[3*h+1] - vl[3*v+1]*hl[3*h];
      assert(d != 0);
      
      dat[(h*sz+v)*3] = Sint16((vl[3*v+1]*hl[3*h+2]-hl[3*h+1]*vl[3*v+2]) / d + 0.5);
      dat[(h*sz+v)*3+1] = Sint16((hl[3*h]*vl[3*v+2]-vl[3*v]*hl[3*h+2]) / d + 0.5);
    }
    
  int minrad = 10000;

  // finally calculate the radius of every stone by taking the smallest difference
  // to all 4 neighbours and half it
  for (int h = 0; h < sz; h++)
    for (int v = 0; v < sz; v++) {
      // FIXME: select a value that comes from the coordinates
      d = 100000;
      if (h > 0) {
        x = dat[(h*sz+v)*3] - dat[((h-1)*sz+v)*3];
	y = dat[(h*sz+v)*3+1] - dat[((h-1)*sz+v)*3+1];
        d = min(d, sqrt(x*x + y*y));
      }
      if (h < sz-1) {
        x = dat[(h*sz+v)*3] - dat[((h+1)*sz+v)*3];
	y = dat[(h*sz+v)*3+1] - dat[((h+1)*sz+v)*3+1];
        d = min(d, sqrt(x*x + y*y));
      }
      if (v > 0) {
        x = dat[(h*sz+v)*3] - dat[(h*sz+v-1)*3];
	y = dat[(h*sz+v)*3+1] - dat[(h*sz+v-1)*3+1];
        d = min(d, sqrt(x*x + y*y));
      }
      if (v < sz-1) {
        x = dat[(h*sz+v)*3] - dat[(h*sz+v+1)*3];
	y = dat[(h*sz+v)*3+1] - dat[(h*sz+v+1)*3+1];
        d = min(d, sqrt(x*x + y*y));
      }
      dat[(h*sz+v)*3+2] = int((d-3)/2);
      if ((d-3)/2 < minrad) minrad = int((d-3)/2);
    }
    
  // the thickness of lines is calculates using the smallest radius possible
  thickness = minrad / 5;
  if (thickness < 1) thickness = 1;
  
  delete [] vl;
  delete [] hl;
}
    
Sint16 BoardLines::xPos(const Vertex &v) const {
  assert(dat != NULL);
  assert((v.row() < size) && (v.col() < size));
  
  return dat[((size-v.row()-1)*size+v.col())*3];
}

Sint16 BoardLines::yPos(const Vertex &v) const {
  assert(dat != NULL);
  assert((v.row() < size) && (v.col() < size));
  
  return dat[((size-v.row()-1)*size+v.col())*3 + 1];
}
Sint16 BoardLines::rad(const Vertex &v) const {
  assert(dat != NULL);
  assert((v.row() < size) && (v.col() < size));
  
  return dat[((size-v.row()-1)*size+v.col())*3 + 2];
}

void BoardLines::setBoardSize(istream &in, Uint8 sz) {
  if (lines != NULL) delete lines;
  if (dat != NULL) {
    delete [] dat;
    dat = NULL;
  }
  
  size = sz;

  Uint16 w, h;
  Sint16 x, y;
  
  in.read(&w, 2);
  in.read(&h, 2);
  in.read(&x, 2);
  in.read(&y, 2);
  
  imagePos = Point(x, y);
  
  lines = new Surface(w, h, Color::getTrueColor()?Pixelformat(32):Pixelformat(8));
  lines->loadSprite(in);
}

void BoardLines::draw(Surface &dest, const Rect &r) {
  assert((dat != NULL) || (lines != NULL));
  
  dest.setClipRect(r);
 
  if (lines != NULL) {
    lines->blit(dest, imagePos);
  } else {
    int tm = thickness >> 1;
    for (int l = 0; l < size; l++) {
      for (int t = 0; t < thickness; t++) {
        dest.line(Point(dat[(0*size+l)*3]-tm+t, dat[(0*size+l)*3+1]),
                  Point(dat[((size-1)*size+l)*3]-tm+t, dat[((size-1)*size+l)*3+1]), 
  		  lineColor);
        dest.line(Point(dat[(l*size+0)*3], dat[(l*size+0)*3+1]-tm+t),
                  Point(dat[(l*size+size-1)*3], dat[(l*size+size-1)*3+1]-tm+t),
		  lineColor);
      }
    }             
  }
}