/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



#include "datafile.h"
#include "pixelformat.h"

#include <fstream>
 
Datafile::Datafile(string filename) {
  fileName = filename;

  ifstream in(filename.c_str());
  char tmp1[5];
  Uint16 tmp2, tmp3;
  
  // check if file was opened successfully
  if (!in.good()) assert(false); // FIXME: exception
  
  // check filemarker for right file
  in.read(&tmp1[0], 5); 
  if ((tmp1[0] != 'S') ||
      (tmp1[1] != 'D') ||
      (tmp1[2] != 'L') ||
      (tmp1[3] != 'G') ||
      (tmp1[4] != 'o')) {
    // FIXME
    assert(false);
  }

  // check version
  in.read(&tmp1[0], 1); 
  if (tmp1[0] != 0) assert(false); // FIXME: exception
  
  // read description
  in >> description;
  
  // read resolution
  in.read(&xResolution, 2); in.read(&yResolution, 2);
  
  bool trueColor;
  in.read(&trueColor, 1);
  
  Color::setTrueColor();
  
  if (!trueColor) {
    // read palette
    SDL_Color *pal = new SDL_Color[256];
    for (int i = 0; i < 255; i++)
    pal[i] = SDL_Color(Color(in));
  
    Color::setPalette(pal);
  }
  
  background = new Background(in);
  
  // error protection
  in >> tmp1[0]; 
  if (tmp1[0] != 'B') assert(false); // FIXME: exception
  
  menu = new Menu(in);

  // error protection
  in >> tmp1[0]; if (tmp1[0] != 'M') assert(false); // FIXME: exception
  
  // read the size of mousepointer
  in.read(&tmp2, 2); in.read(&tmp3, 2);
  
  if ((tmp2 != 0) && (tmp3 != 0)) {
    mousePointer = new Surface(tmp2, tmp2, Color::getTrueColor?Pixelformat(32):Pixelformat(8));
    mousePointer->loadSprite(in);
  } else 
    mousePointer = NULL;
  
  // error protection
  in >> tmp1[0]; if (tmp1[0] != 'P') assert(false); // FIXME: exception
  
  // read the size of the fontletters
  in.read(&tmp2, 2); in.read(&tmp3, 2);
  if ((tmp1 != 0) && (tmp2 != 0)) {
    // FIXME
  }
  
  // error protection
  in >> tmp1[0]; if (tmp1[0] != 'F') assert(false); // FIXME: exception
  
  // sounds
  for (int s = 0; s < 3; s++) {
    in >> tmp1[0];
    if (tmp1[0]) {
      // FIXME
    }
  }

  // error protection
  in >> tmp1[0]; if (tmp1[0] != 'S') assert(false); // FIXME: exception
  
  lines = new BoardLines(in);
  
  // error protection
  in >> tmp1[0]; if (tmp1[0] != 'L') assert(false); // FIXME: exception
  
  stones = new Stones(in, lines);

  blackOwner = Color(in);
  whiteOwner = Color(in);

  in >> tmp1[0]; if (tmp1[0] != 'C') assert(false); // FIXME: exception
  
  for (int i = 0; i < 25; i++)
    specialStart[i] = 0;
    
  // FIXME: special board missing
  in.read(&tmp1[0], 1);
  
  int datasize;
  
  while (tmp1[0] != 0) {
    in.read(&datasize, 4);
  
    specialStart[tmp1[0]] = in.tellg();
    
    in.ignore(datasize);
  
    in.read(&tmp1[0], 1);
  }
  
  output = new Surface(xResolution, yResolution, Pixelformat(32));

  board = NULL;
  dirtyArea = Rect::empty;
}

Datafile::~Datafile() {
  if (board != NULL) delete board;
  if (background != NULL) delete background;
  if (menu != NULL) delete menu;
  if (mousePointer != NULL) delete mousePointer;
  if (lines != NULL) delete lines;
  if (stones != NULL) delete stones;
}
    
void Datafile::setBoardSize(Uint8 size) { 

  if (board != NULL) delete board;
  board = new Board(size);

  boardSize = size;
  
  if (specialStart[size] != 0) {
    cout << "  loading special board size " << (int)size << endl;
    ifstream in(fileName.c_str());
    in.seekg(specialStart[size]);
    
    lines->setBoardSize(in, size);
    stones->setBoardSize(in, size);
  } else {
    lines->setBoardSize(size);
    stones->setBoardSize(size);
  }
}
    
void Datafile::drawEmptyBoard() { 
  board->clearBoard();
  
  Rect r = Rect(0, 0, xResolution, yResolution);

  background->draw(*output, r);
  menu->draw(*output, r);
  lines->draw(*output, r);
  
  dirtyArea = dirtyArea.unite(r);
}

void Datafile::updateVertex(const Vertex &v, const Uint8 color, bool mark = false) {
  
  assert((color == Board::EMPTY) || (color == Board::BLACK) || (color == Board::WHITE));
  assert((v.row() < boardSize) && (v.col() < boardSize));
  
  if (color == Board::EMPTY) {
    if (board->getVertex(v) == color) return;
    
    Rect a = stones->getStoneRect(v, (board->getVertex(v) == Board::BLACK));
    
    background->draw(*output, a);
    lines->draw(*output, a);
  
    // the previous action may have destroyed some
    // stones around the actual one (if they do overlal)
    // thats why restore all the stones around the actual position now
  
    if ((v.row() > 0)           && (board->getVertex(v.above()) != Board::EMPTY)) 
      updateVertex(v.above(), board->getVertex(v.above()));
    if ((v.row() < boardSize-1) && (board->getVertex(v.below()) != Board::EMPTY)) 
      updateVertex(v.below(), board->getVertex(v.below()));
    if ((v.col() > 0)           && (board->getVertex(v.left())  != Board::EMPTY)) 
      updateVertex(v.left(),  board->getVertex(v.left()));
    if ((v.col() < boardSize-1) && (board->getVertex(v.right()) != Board::EMPTY)) 
      updateVertex(v.right(), board->getVertex(v.right()));
  
    dirtyArea = dirtyArea.unite(a);
  } else {
    // this part is done even if there is already the same stone there
    // this is necessary to restore stones after removing a neighbour
    // or to remove the last move marker
  
    Uint8 a, b, l, r;
    
    a = b = l = r = Board::EMPTY;
    
    if (v.row() > 0) a = board->getVertex(v.above());
    if (v.row() < boardSize - 1)  b = board->getVertex(v.below());
    if (v.col() > 0) l = board->getVertex(v.left());
    if (v.col() < boardSize - 1) r = board->getVertex(v.right());
  
    dirtyArea = dirtyArea.unite(stones->putStone(*output, v, color == 1, a, r, b, l));
  
    if (mark) stones->setMarker(*output, v);
  }
}
    
void Datafile::updateBoard(const Game *newGame) {
  const Board *newBoard = newGame->getBoard();

  // remove old mark (if there is one)
  if (marked.isValid()) updateVertex(marked, board->getVertex(marked));

  for (int y = 0; y < board->getSize(); y++)
    for (int x = 0; x < board->getSize(); x++) {
      Vertex v(y, x);
      if (newBoard->getVertex(v) != board->getVertex(v)) {
        updateVertex(v, newBoard->getVertex(v));
	board->setVertex(v, newBoard->getVertex(v));
      }
    }
    
  marked = newGame->getLast();
  
  // put new mark (if there is one)
  if (marked.isValid()) updateVertex(marked, board->getVertex(marked), true);
}
    
void Datafile::pressButton(Uint8 num) { 
  dirtyArea = dirtyArea.unite(menu->pressButton(*output, num));
}

void Datafile::releaseButton(Uint8 num) { 
  // because the menu can only press but not release image buttons we do the following:
  // get the area of the press sprite
  // put background onto this area, redraw the menu there
  Rect r = menu->getButtonImageRect(num);
  
  background->draw(*output, r);
  menu->draw(*output, r);
  
  dirtyArea = dirtyArea.unite(r);
}
    
void Datafile::repaint() {
  Rect r = Rect(0, 0, xResolution, yResolution);
  
  background->draw(*output, r);
  lines->draw(*output, r);
  menu->draw(*output, r);
  
  dirtyArea = dirtyArea.unite(r);

  for (int y = 0; y < board->getSize(); y++)
    for (int x = 0; x < board->getSize(); x++) {
      Vertex v(y, x);
      updateVertex(v, board->getVertex(v));
    }
    
  // put new mark (if there is one)
  if (marked.isValid()) updateVertex(marked, board->getVertex(marked), true);
}
    
void Datafile::drawOwnership(const Board &own) {
  Surface s(xResolution, yResolution, Pixelformat(32));
  
  s.fill(Color(0, 0, 0, 255));
  
  for (int x = 0; x < boardSize; x++)
    for (int y = 0; y < boardSize; y++)
      if (own.getVertex(Vertex(y, x)) != Board::EMPTY) {
        Vertex v(y, x);
	Color c;
	if (own.getVertex(Vertex(y, x)) == Board::BLACK)
  	  c = blackOwner;
	else
	  c = whiteOwner;
	  
	c.setAlpha(128);
        s.filledCircle(Point(lines->xPos(v), lines->yPos(v)), lines->rad(v)*1.5, c);
      }
      
  s.blit(*output, Point(0, 0));
  dirtyArea = dirtyArea.unite(Rect(0, 0, xResolution, yResolution));
}
void Datafile::removeOwnership() {
  repaint();
}