/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



/**
 * this class reads a datafile and returns the necessary data for
 * all things. this object is there to update the go board display
 * whenever needed and to return sounds and other things
 * attention! no update is called after changes, when changes are
 * made it is possible that things start to flicker so the best is
 * to let this object draw onto a surface that is invisible
 * and then update regions that have changed onto the screen
 * the cliprect of the targetsurface is automatically set so
 * that only the changed parts are within the rect
 */
 
#ifndef DATAFILE__H_
#define DATAFILE__H_
 
#include "vertex.h"
#include "surface.h"
#include "board.h"
#include "color.h"
#include "boardlines.h"
#include "background.h"
#include "menu.h"
#include "stones.h"
#include "point.h"
#include "game.h"
 
#include <SDL/SDL.h>
#include <string>
#include <iostream>

class Datafile {

  public:
  
    /**
     * opens the datafile and automatically selects the 19 sized board
     * a surface of the required size is automatically renerated in
     * memory
     */
    Datafile(string filename);
    
    ~Datafile();
    
    /**
     * selects another boardsize
     */
    void setBoardSize(Uint8 size);
    
    /**
     * returns the vertex the given coordinates are situated on
     */
    Vertex getVertex(const Point &p) const { return stones->getVertex(p); }

    /**
     * clears the board
     */    
    void drawEmptyBoard();
    
    /**
     * puts a stone onto the board using the constants from board for color
     * if mark is true the stone will be marked as the last move made
     * this function also clears away stones on the board
     */
    void updateVertex(const Vertex &v, const Uint8 color, bool mark = false);
    
    /**
     * updates the board on screen to correspond with the game given
     */
    void updateBoard(const Game *newBoard);
    
    void pressButton(Uint8 number);
    void releaseButton(Uint8 number);
    
    /**
     * repaints everything
     */
    void repaint();
    
    /**
     * draws shaded ownership over the board
     */
    void drawOwnership(const Board &own);
    void removeOwnership();
    
    Uint16 getXResolution() const { return xResolution; }
    Uint16 getYResolution() const { return yResolution; }
    Uint8 getBitsPerPixel() const { return 32; }
    
    const Rect &getButtonRect(Uint8 nr) const { return menu->getButtonPushRect(nr); }
    const string& getDescription() { return description; }
    
    Surface *getSurface() { return output; }
    
    /**
     * returns the rectangle that has been changed 
     * since the last call to this function
     */
    const Rect getUpdateRect() { 
      Rect r = dirtyArea;
      dirtyArea = Rect::empty;
      return r; }
     
  private:
  
    /**
     * all output will occure onto this surface
     */
    Surface *output;
    
    /**
     * represents the actual visible board
     */
    Board *board;
    
    /**
     * the description within the datafile
     */
    string description;
    
    /**
     * the resolution the graphics are designed for
     */
    Uint16 xResolution;
    Uint16 yResolution;

    Background *background;
    BoardLines *lines;
    Menu *menu;
    Stones *stones;
    
    /**
     * the mousepointer
     */
    Surface *mousePointer;
    
    /**
     * !FIXME: font is missing
     */
     
    /**
     * !FIXME: sounds is missing
     */
     
    /**
     * the ownership diagram is drawn using this color
     */
    Color blackOwner;
    Color whiteOwner;
    
    /**
     * array with startingpositions for special boards
     * boards bigger than 25 are not allowed
     */
    streampos specialStart[MAXBOARDSIZE + 1];
    
    /**
     * the actual boardsize
     */
    Uint8 boardSize;    
    
    Rect dirtyArea;
    
    string fileName;
    
    /**
     * the vertex of the marked stone
     */
    Vertex marked;
};

#endif