/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



#include "game.h"

#include <assert.h>

Game::Game(const Options &opt) {

  // check boardsize
  assert((opt.getBoardsize() >= 5) && (opt.getBoardsize() <= 25));
  
  // controller must be a program
  assert(opt.getController() > 0);
  assert(opt.getComputer(opt.getController()) != "");
  
  // check if the blackplayer is either human or has a program
  // defined
  assert((opt.getBlackplayer() == 0) || 
         (opt.getComputer(opt.getBlackplayer()) != ""));
  assert((opt.getWhiteplayer() == 0) || 
         (opt.getComputer(opt.getWhiteplayer()) != ""));
	 
  programs[0] = programs[1] = programs[2] = NULL;
	 
  // first start the controller
  controller = new GTP(opt.getComputer(opt.getController()));
  programs[0] = controller;
  
  int pcount = 1;
  
  // now blackplayer
  if (opt.getBlackplayer() != 0) {
    if (opt.getBlackplayer() != opt.getController()) {
      programB = new GTP(opt.getComputer(opt.getBlackplayer()));
      programs[pcount++] = programB;
    } else
      programB = controller;
  } else {
    programB = NULL;
  }
    
  // finally the white player
  if (opt.getWhiteplayer() != 0) {
    if (opt.getWhiteplayer() == opt.getController())
      programW = controller;
    else if (opt.getWhiteplayer() == opt.getBlackplayer())
      programW = programB;
    else {
      programW = new GTP(opt.getComputer(opt.getWhiteplayer()));
      programs[pcount++] = programW;
    }
  } else {
    programW = NULL;
  }
  
  for (int i = 0; i < 3; i++)
    if (programs[i] != NULL)
      programs[i]->boardsize(opt.getBoardsize());
  
  lastCommand = -1;
  
  size = opt.getBoardsize();
  board = new Board(size);
}

Game::~Game() {
  for (int i = 0; i < 3; i++)
    if (programs[i] != NULL) {
      programs[i]->quit();
      delete programs[i];
    }
}

bool Game::blackM(const Vertex &v) {
  // let the controller check if the move is legal
  if (!controller->cmd_blackLegal(v)) return false;
  
  // not inform all programs of the move
  for (int i = 0; i < 3; i++)
    if (programs[i] != NULL)
      programs[i]->moveBlack(v);
      
  updateBoard(v);
  
  last = v;
  return true;
}
void Game::blackP() {
  last = Vertex();
}
void Game::blackC() {
  assert(programB != NULL);
  assert(lastCommand == -1);
  lastCommand = programB->cmd_genmoveBlack();
  lastProgram = programB;
  lastBlack = true;
}


bool Game::whiteM(const Vertex &v) {
  // let the controller check if the move is legal
  if (!controller->cmd_whiteLegal(v)) return false;
  
  // not inform all programs of the move
  for (int i = 0; i < 3; i++)
    if (programs[i] != NULL)
      programs[i]->moveWhite(v);
      
  updateBoard(v);
  
  last = v;
  return true;
}
void Game::whiteP() {
  last = Vertex();
}
void Game::whiteC() {
  assert(programW != NULL);
  assert(lastCommand == -1);
  lastCommand = programW->cmd_genmoveWhite();
  lastProgram = programW;
  lastBlack = false;
}

bool Game::computerFinished() {
  if (lastCommand == -1) return true;
  Vertex v;
  
  if (lastProgram->res_genmove(lastCommand, v)) {
    if (v.isValid()) {
      for (int i = 0; i < 3; i++)
        if ((programs[i] != NULL) && (programs[i] != lastProgram))
	  if (lastBlack)
	    programs[i]->moveBlack(v);
	  else
            programs[i]->moveWhite(v);
      updateBoard(v);
    } 
    last = v;
    lastCommand = -1;
    return true;
  } return false;
}

void Game::updateBoard(const Vertex  &v) {
  Uint8 nColor = controller->cmd_getVertex(v);
  
  if (nColor != board->getVertex(v)) {
    board->setVertex(v, nColor);
    if (v.row() > 0) updateBoard(Vertex(v.row()-1, v.col()));
    if (v.col() > 0) updateBoard(Vertex(v.row(), v.col()-1));
    if (v.row() < size-1) updateBoard(Vertex(v.row()+1, v.col()));
    if (v.col() < size-1) updateBoard(Vertex(v.row(), v.col()+1));
  }
}

void Game::newGame() {
  for (int i = 0; i < 3; i++)
    if (programs[i] != NULL)
      programs[i]->boardsize(size);
  
  board->clearBoard();
}

void Game::loadSGF(string filename) {
  if (programB != NULL) programB->loadSGF(filename);
  if (programW != NULL) programW->loadSGF(filename);
  GTP *program = (programB != NULL) ? programB : programW;
  
  // load the whole board now
  // FIXME: ask gngo for the actual boardsize
  
  for (int y = 0; y < 19; y++)
    for (int x = 0; x < 19; x++)
      board->setVertex(Vertex(y, x), program->cmd_getVertex(Vertex(y, x)));
}