/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



/**
 * implements the GTP-Protocoll with the given program
 * the object creates a thread that polls the output of the
 * program and puts the output into a queue where it can be taken
 * from evaluation functions
 *
 * be careful to call the evaluation funtions in the same
 * order as you called the command functions because the output is
 * not reorderes other orders will confuse the functions
 *
 * there exist two versions of every function: the fist one (cmd_*) commands
 * the GTP-program, the 2nd one (res_*) evaluates the result it will return
 * true if the result was available the result itself will be given through
 * reference parameters
 *
 * some functions do not have a res_* part, this means that this functions
 * wait for the output by themselves (becaues they are normally fast enough)
 *
 * you mustn't call this kind of function while you are waiting for another
 * result
 *
 * finally there is a kind of function without prefix these can be called whenever
 * you want
 */
#ifndef GTP__H_
#define GTP__H_ 

#include "duplexpipe.h"
#include "vertex.h"

#include <SDL/SDL.h>
#include <SDL/SDL_thread.h>

#include <map>

typedef struct {
  string cmdResult;
  bool   cmdForget;
} GTPCommand;

class GTP {

  public:
  
    /**
     * connects to the given program
     */
    GTP(const string &command);
    
    ~GTP();
  
    /**
     * lets the computer generate a white move
     */
    int cmd_genmoveWhite();
    bool res_genmove(int command, Vertex &v);
    /**
     * lets the computer generate a black move
     */
    int cmd_genmoveBlack();

    /**
     * puts a black stone at the given Vertex
     */
    void moveBlack(const Vertex &v);

    /**
     * puts a white stone at the given vertes
     */
    void moveWhite(const Vertex &v);

    /**
     * returns true if the given move is legal for white
     */
    bool cmd_whiteLegal(const Vertex &v);

    /**
     * returns true if the given move is legal for black
     */
    bool cmd_blackLegal(const Vertex &v); 

    /**
     * sets the boardsize of the go program
     */
    void boardsize(Uint8 sz);

    /**
     * returns the color for the given vertex (0 = empty, 1 = black, 2 = white)
     */
    Uint8 cmd_getVertex(const Vertex &v); 
  
    /**
     * return true, if still connected to the go program
     */
    bool connected();
  
    /**
     * quits the program
     */
    void quit();
    
    /**
     * loads an sgf-file
     */
    void loadSGF(string filename);
  
  private:
    
    DuplexPipe *go;

    /**
     * the current results of active commands
     */
    map<int, GTPCommand> commands;

    /**
     * the current command number
     */
    Uint16 commandNumber;
    
    /** 
     * temp functionto test the poller
     */
    void callPoller();
    
    /**
     * a function that gets the output of the pipe
     */
    void getOutput();
};

#endif