/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



#include "menu.h"
#include "color.h"

#include <assert.h>
 
Menu::Menu(istream &in) {
  in.read(&menuOnBackground, 1);

  // there can be only a menu if it is not already on backgroundpicture
  if (!menuOnBackground) {
    Uint16 w, h;
    in.read(&w, 2);
    in.read(&h, 2);
    
    if ((w != 0) && (h != 0)) {
      Sint16 x, y;
      in.read(&x, 2);
      in.read(&y, 2);
      menuPos = Point(x, y);
      
      image = new Surface(w, h, Color::getTrueColor()?Pixelformat(32):Pixelformat(8));
      image->loadSprite(in); 
    } else {
      image = NULL;
    }
  } else image = NULL;
  
  // now load the 6 buttons
  for (int b = 0; b < MAXBUTTONS; b++) {
    Uint16 w, h;
    Sint16 x, y;
    
    in.read(&w, 2);
    in.read(&h, 2);
    in.read(&x, 2);
    in.read(&y, 2);
    buttons[b] = Rect(x, y, w, h);
    
    // allow only pictures for the buttons if there exists
    // a menu image
    if (menuOnBackground || (image != NULL)) {
      in.read(&w, 2);
      in.read(&h, 2);
      if ((w != 0) && (h != 0)) {
        in.read(&x, 2);
	in.read(&y, 2);
	positions[b] = Point(x, y);
	
	press[b] = new Surface(w, h, Color::getTrueColor()?Pixelformat(32):Pixelformat(8));
	press[b]->loadSprite(in);
      } else {
        press[b] = NULL;
      }
    } else {
      buttonColors[0][b] = Color(in);
      buttonColors[1][b] = Color(in);
      buttonColors[2][b] = Color(in);
      buttonColors[3][b] = Color(in);
    }
  }
  
  // and finally the starts of the 4 status lines
  for (int l = 0; l < 4; l++) {
    Sint16 x, y;
    in.read(&x, 2);
    in.read(&y, 2);
    
    statusStart[l] = Point(x, y);
  }
  
  font = new TTFont("timrom.ttf", 30);
  textArea[0] = textArea[1] = Rect::empty;
}

Menu::~Menu() {
  if (image != NULL) delete image;
  
  for (int b = 0; b < MAXBUTTONS; b++)
    if (press[b] != NULL) delete press[b];
}
    
void Menu::draw(Surface &dst, const Rect &r) {
  // if the menu is already on the backgroundimage we don't need to do anything
  dst.setClipRect(r);
  
  if (!menuOnBackground) {
    if (image == 0) {
      for (int b = 0; b < MAXBUTTONS; b++) 
        draw3dButton(dst, b, false);
    } else {
      image->blit(dst, menuPos);
    }
  }
}
    
const Rect Menu::pressButton(Surface &dst, const Uint8 nr) {
  assert(nr < MAXBUTTONS);
  dst.setClipRect(getButtonImageRect(nr));
  if (press[nr] == NULL) {
    if ((image == NULL) && (!menuOnBackground))
      draw3dButton(dst, nr, true);
//      dst.bar(buttons[nr], Color(250, 50, 50));
    return buttons[nr];
  } else {
    dst.clearClipRect();
    press[nr]->blit(dst, positions[nr]);
    return Rect(positions[nr].getX(), positions[nr].getY(),
                press[nr]->width(), press[nr]->height());
  }
}

const Rect &Menu::getButtonPushRect(const Uint8 nr) const {
  assert(nr < MAXBUTTONS);
  return buttons[nr];
}

const Rect Menu::getButtonImageRect(const Uint8 nr) const {
  assert(nr < MAXBUTTONS);
  if (press[nr] != NULL)
    return Rect(positions[nr].getX(), positions[nr].getY(),
                press[nr]->width(), press[nr]->height());
  else
    return buttons[nr];
}

void Menu::draw3dButton(Surface &dst, const Uint8 nr, const bool press) {
  assert(nr < MAXBUTTONS);
  
  dst.bar(buttons[nr], buttonColors[0][nr]);
  
  if (!press) {
    for (int l = 0; l < 2; l++) {
      dst.line(Point(buttons[nr].getUpperLeft().getX()+l, buttons[nr].getUpperLeft().getY()+l),
               Point(buttons[nr].getLowerRight().getX()-l, buttons[nr].getUpperLeft().getY()+l), 
	       buttonColors[1][nr]);
      dst.line(Point(buttons[nr].getUpperLeft().getX()+l, buttons[nr].getUpperLeft().getY()+l),
               Point(buttons[nr].getUpperLeft().getX()+l, buttons[nr].getLowerRight().getY()-l),
	       buttonColors[1][nr]);
      dst.line(Point(buttons[nr].getLowerRight().getX()-l, buttons[nr].getLowerRight().getY()-l),
               Point(buttons[nr].getUpperLeft().getX()+l, buttons[nr].getLowerRight().getY()-l), 
	       buttonColors[2][nr]);
      dst.line(Point(buttons[nr].getLowerRight().getX()-l, buttons[nr].getLowerRight().getY()-l),
               Point(buttons[nr].getLowerRight().getX()-l, buttons[nr].getUpperLeft().getY()+l),
	       buttonColors[2][nr]);
    }
  } else {
    for (int l = 0; l < 2; l++) {
      dst.line(Point(buttons[nr].getUpperLeft().getX()+l, buttons[nr].getUpperLeft().getY()+l),
               Point(buttons[nr].getLowerRight().getX()-l, buttons[nr].getUpperLeft().getY()+l), 
	       buttonColors[2][nr]);
      dst.line(Point(buttons[nr].getUpperLeft().getX()+l, buttons[nr].getUpperLeft().getY()+l),
               Point(buttons[nr].getUpperLeft().getX()+l, buttons[nr].getLowerRight().getY()-l),
	       buttonColors[2][nr]);
      dst.line(Point(buttons[nr].getLowerRight().getX()-l, buttons[nr].getLowerRight().getY()-l),
               Point(buttons[nr].getUpperLeft().getX()+l, buttons[nr].getLowerRight().getY()-l), 
	       buttonColors[1][nr]);
      dst.line(Point(buttons[nr].getLowerRight().getX()-l, buttons[nr].getLowerRight().getY()-l),
               Point(buttons[nr].getLowerRight().getX()-l, buttons[nr].getUpperLeft().getY()+l),
	       buttonColors[1][nr]);
    }
  }
  
  Point m = Point((buttons[nr].getLowerRight().getX() + buttons[nr].getUpperLeft().getX()) / 2 + ((press) ? 1 : 0),
                  (buttons[nr].getLowerRight().getY() + buttons[nr].getUpperLeft().getY()) / 2 + ((press) ? 1 : 0));
		  
  string text;
  
  switch (nr) {
  case 0: text = "Quit"; break;
  case 1: text = "Pass"; break;
  case 2: text = "New"; break;
  case 3: text = "Done"; break;
  case 4: text = "Options"; break;
  case 5: text = "About"; break;
  }
  
  dst.text(m, text, buttonColors[3][nr], *font, 0, 0);
}

const Rect Menu::setText(const Uint8 disp, const string &text) {
  return Rect::empty;
}