/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



#ifndef MENU__H_
#define MENU__H_

#include "surface.h"
#include "rect.h"
#include "point.h"
#include "ttfont.h"

#include <SDL/SDL.h>

#include <iostream.h>
#include <string>

/**
 * the number of buttons the menu contains
 */
#define MAXBUTTONS 6
 
/**
 * This class is responsible for the menu.
 * It loads the menu from the stream, draws is onto a surface, presses 
 * and releases buttons, handles the size and activation area of buttons
 *
 * If you do not specify a menu image then the buttons are drawn using
 * internal functions and colors these can not be changed
 */
class Menu {

  public:
    /**
     * Loads all necessary data from the stream.
     * The exact format of this data is detailed in the file FileFormat.txt in
     * the root directory.
     */
    Menu(istream &);
    
    /**
     * Releases all memory.
     */
    ~Menu();

    /**
     * Draws the menu with all buttons unpressed.
     * This is done only in the area specified by r so that you can
     * repaint only a part of the whole menu. This function assumes that
     * there is background painted within this area of the screen.
     */
    void draw(Surface &dst, const Rect &r);
    
    /**
     * Changes the appearance of the given button to be pressed.
     * this function assumes that there is the unpressed button shown
     * on dst. The changed area is returned.
     */
    const Rect pressButton(Surface &dst, const Uint8 nr);
    
    /**
     * Returns the activation area of this button. This is the area
     * where the button can be pressed. It is neccessary to
     * distinguish this from the image area to have a border around the
     * actual button.
     */
    const Rect &getButtonPushRect(const Uint8 nr) const;
    
    /**
     * Returns the area of the sprite for this button.
     * It is the same as pressButton but without painting
     * the button itself.
     */
    const Rect getButtonImageRect(const Uint8 nr) const;
    
    /**
     * sets the text for one of the two displays
     */
    const Rect setText(const Uint8 disp, const string &text);
    
  private:
    
    /// This variable is true if the menu is already painted onto the backgroundimage.
    bool menuOnBackground;
    
    /// The sprite for the menu image, this is NULL if there is no menu image.
    Surface *image;
    
    /// The position of the menuimage on screen if there is one.
    Point menuPos;
    
    /// The sprites to change the appearance of the onpressed button into a pressed one
    Surface *press[MAXBUTTONS];
    
    /// activation areas of the singe buttons
    Rect buttons[MAXBUTTONS];
    
    /// positions for the sprites in press[]
    Point positions[MAXBUTTONS];
    
    /// positions for the 4 text lines
    Point statusStart[4];
    
    /// if there are no images this colors are used to draw the
    /// buttons
    Color buttonColors[4][MAXBUTTONS];
    
    /// draws a 3d button
    void draw3dButton(Surface &dst, const Uint8 nr, const bool press);
    
    /// the font used for the button labels (fixed size and face for the moment
    TTFont *font;
    
    /// represents the area of the actual visible text
    Rect textArea[2];
};

#endif