/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



#include "surface.h"

#include <assert.h>

Surface::Surface() { sdlSurface = NULL; }

Surface::Surface(const int w, const int h, const Pixelformat &f) {
  
  sdlSurface = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, f.bpp()*8, 
                                    f.rMask(), f.gMask(), f.bMask(), f.aMask());
  if (sdlSurface != NULL) {
    format = new Pixelformat(sdlSurface->format->BytesPerPixel,
                             sdlSurface->format->Rmask,
			     sdlSurface->format->Gmask,
			     sdlSurface->format->Bmask,
			     sdlSurface->format->Amask);
/*			     
    for (int y = 0; y < h; y++)
      for (int x = 0; x < w; x++)
        putpixel(Point(x, y), Color(255, 255, 255, 128));*/
//    for (int b = 0; b < w*h; b++) ((Uint8*)sdlSurface->pixels)[b] = 255;
  } else
    format = NULL;
  
  maxCoord = Point(w - 1, h - 1);
}

Surface::Surface(const Surface& surf) {
  sdlSurface = SDL_CreateRGBSurface(surf.sdlSurface->flags,
				    surf.sdlSurface->w,
				    surf.sdlSurface->h,
				    surf.sdlSurface->format->BitsPerPixel,
				    surf.sdlSurface->format->Rmask,
				    surf.sdlSurface->format->Gmask,
				    surf.sdlSurface->format->Bmask,
				    surf.sdlSurface->format->Amask);
  if (sdlSurface == NULL) {
    // FIXME: throw exceptin
    exit(2);
  }

  SDL_BlitSurface(surf.sdlSurface,0,sdlSurface,0);

  if (sdlSurface != NULL)
    format = new Pixelformat(sdlSurface->format->BytesPerPixel,
                             sdlSurface->format->Rmask,
			     sdlSurface->format->Gmask,
			     sdlSurface->format->Bmask,
			     sdlSurface->format->Amask);
  else
    format = NULL;
  
  maxCoord = Point(sdlSurface->w - 1, sdlSurface->h - 1);
}

Surface::Surface(SDL_Surface *surf) {
  sdlSurface = surf;

  if (sdlSurface == NULL) {
    // FIXME: throw exceptin
    exit(2);
  }

  format = new Pixelformat(sdlSurface->format->BytesPerPixel,
                           sdlSurface->format->Rmask,
			   sdlSurface->format->Gmask,
			   sdlSurface->format->Bmask,
			   sdlSurface->format->Amask);
  
  maxCoord = Point(sdlSurface->w - 1, sdlSurface->h - 1);
}
 
Surface::~Surface() { 
  if (sdlSurface != NULL) SDL_FreeSurface(sdlSurface);
  if (format != NULL) delete format;
}

void Surface::blit(Surface &dst, const Point &p) const {
  blit(dst, Rect(0, 0, width(), height()), p);
}

void Surface::blit(Surface &dst, const Rect& srcArea, const Point &dstPoint) const {
  SDL_Rect srcA, dstA;
  int h;
  
  // first look if there is something at all visible
  if ((dstPoint.getX() > dst.sdlSurface->clip_maxx) || 
      (dstPoint.getX()+srcArea.width() < dst.sdlSurface->clip_minx) ||
      (dstPoint.getY() > dst.sdlSurface->clip_maxy) || 
      (dstPoint.getY()+srcArea.height() < dst.sdlSurface->clip_miny)) return;
     
  srcA = srcArea.toSDL();
  dstA = srcArea.toSDL();
  dstA.x += dstPoint.getX() - srcArea.getUpperLeft().getX();
  dstA.y += dstPoint.getY() - srcArea.getUpperLeft().getY();
  
  if (dstA.x < dst.sdlSurface->clip_minx) {
    h = dst.sdlSurface->clip_minx-dstA.x;
    srcA.x += h;
    srcA.w -= h;
    dstA.w -= h;
    dstA.x = dst.sdlSurface->clip_minx;
  }
  
  if (dstA.y < dst.sdlSurface->clip_miny) {
    h = dst.sdlSurface->clip_miny-dstA.y;
    srcA.y += h;
    srcA.h -= h;
    dstA.h -= h;
    dstA.y = dst.sdlSurface->clip_miny;
  }
  
  if (dstA.x + dstA.w > dst.sdlSurface->clip_maxx) {
    h = dstA.x + dstA.w - dst.sdlSurface->clip_maxx;
    srcA.w -= h;
    dstA.w -= h;
  }
  
  if (dstA.y + dstA.h > dst.sdlSurface->clip_maxy) {
    h = dstA.y + dstA.h - dst.sdlSurface->clip_maxy;
    srcA.h -= h;
    dstA.h -= h;
  }
  
  SDL_BlitSurface(sdlSurface, &srcA, dst.sdlSurface, &dstA);
}

void Surface::putpixel(const Point& p, const Color& c) {
  assert(sdlSurface != NULL);
  
  if (_mustlock()) {
    if (!_lock()) return;
    _putpixel(p, c);
    _unlock();
  } else 
    _putpixel(p, c);
}  

Color Surface::getpixel(const Point &p) {
  return _getpixel(p);
}


void Surface::fill(const Color &color) {
  bar(Rect(Point(), maxCoord), color);
}

void Surface::bar(const Rect &r, const Color &c) {
  assert(sdlSurface != NULL);
  
  if (_mustlock()) (_lock());
  
  // FIXME: optimize
  for (int x = r.getUpperLeft().getX(); x <= r.getLowerRight().getX(); x++)
    for (int y = r.getUpperLeft().getY(); y <= r.getLowerRight().getY(); y++)
      putpixel(Point(x, y), c);

  if (_mustlock()) (_unlock());
}

void Surface::line(const Point &p1, const Point &p2, const Color &c) {
  assert(sdlSurface != NULL);
  
  if (_mustlock()) (_lock());
  
  Sint16 dx, dy, sdx, sdy, x, y, px, py;
  
  dx = p2.getX() - p1.getX();
  dy = p2.getY() - p1.getY();
  
  sdx = (dx < 0) ? -1 : 1;
  sdy = (dy < 0) ? -1 : 1;
  
  dx = sdx * dx + 1;
  dy = sdy * dy + 1;
  
  x = y = 0;
  
  px = p1.getX();
  py = p1.getY();
  
  if (dx >= dy) {
    for (x = 0; x < dx; x++) {
      _putpixel(Point(px, py), c);
      y += dy;
      if (y >= dx) {
        y -= dx;
	py += sdy;
      }
      px += sdx;
    }
  } else {
    for (y = 0; y < dy; y++) {
      _putpixel(Point(px, py), c);
      x += dx;
      if (x >= dy) {
        x -= dy;
	px += sdx;
      }
      py += sdy;
    }
  }
  
  if (_mustlock()) (_unlock());
}

void Surface::filledCircle(const Point &p, const Uint16 r, const Color &c) {
  assert(sdlSurface != NULL);
  
  Sint16 cx = 0, cy = r, df = 1-r, d_e = 3, d_se= -2*r+5;
  
  do {
    line(Point(p.getX()-cx, p.getY()+cy), Point(p.getX()+cx, p.getY()+cy), c);
    line(Point(p.getX()-cx, p.getY()-cy), Point(p.getX()+cx, p.getY()-cy), c);
    line(Point(p.getX()-cy, p.getY()-cx), Point(p.getX()+cy, p.getY()-cx), c);
    line(Point(p.getX()-cy, p.getY()+cx), Point(p.getX()+cy, p.getY()+cx), c);
    
    if (df < 0) {
      df += d_e;
      d_e += 2;
      d_se += 2;
    } else {
      df += d_se;
      d_e += 2;
      d_se += 4;
      cy--;
    }
    cx++;
  } while (cx <= cy);
}

void Surface::setClipRect(const Rect &r) {
  SDL_SetClipping(sdlSurface, r.getUpperLeft().getY(), r.getUpperLeft().getX(),
                  r.getLowerRight().getY(), r.getLowerRight().getX());
}

void Surface::clearClipRect() {
  SDL_SetClipping(sdlSurface, 0, 0, 0, 0);
}

void Surface::loadSprite(istream &in, Color key) {
  if (!Color::getTrueColor()) 
    SDL_SetColors(sdlSurface, Color::getPalette(), 0, 256);
  
  // fill everything with transparent color
  fill(key);
  
  int y = 0, x = 0;
  unsigned char counter;
  
  while (y < height()) {
    // zeros, skip
    in.read(&counter, 1);
    x += counter;
    while (x >= width()) {
      x -= width();
      y ++;
    }
    if (y >= height()) break;
    // ones, read
    in.read(&counter, 1);
    while (counter > 0) {
      putpixel(Point(x, y), Color(in));
      x++;
      if (x >= width()) {
        x -= width();
	y ++;
      }
      counter --;
    }
  }
      
  SDL_SetColorKey(sdlSurface, SDL_SRCCOLORKEY, format->mapToPixel(key));
}

void Surface::loadImage(istream &in) {
  if (!Color::getTrueColor()) {
    int i = SDL_SetColors(sdlSurface, Color::getPalette(), 0, 256);
    i = i;
  }
  for (int y = 0; y < height(); y++)
    for (int x = 0; x < width(); x++)
      putpixel(Point(x, y), Color(in));	
}

void Surface::text(const Point &p, const string &text, const Color &c, TTFont &f, const Sint8 vDir, const Sint8 hDir) {
  Surface *s = f.render(text, c);

  s->blit(*this, Point(p.getX() - ((hDir < 0) ? 0 : ((hDir == 0) ? s->width()/2 : s->width())),
                       p.getY() - ((vDir > 0) ? 0 : ((vDir == 0) ? s->height()/2 : s->height()))));
}

inline bool Surface::_mustlock() { return SDL_MUSTLOCK(sdlSurface); }
inline bool Surface::_lock() { return SDL_LockSurface(sdlSurface) > 0; }
inline void Surface::_unlock() { SDL_UnlockSurface(sdlSurface); }

void Surface::_putpixel(const Point &p, const Color &c) {
  if ((p.getX() >= sdlSurface->clip_minx) && (p.getX() <= sdlSurface->clip_maxx) &&
      (p.getY() >= sdlSurface->clip_miny) && (p.getY() <= sdlSurface->clip_maxy)) {
      
    Uint8 *i;
    
    Uint32 pix = format->mapToPixel(c);

    switch (format->bpp()) {
      case 1:
        *((Uint8 *)sdlSurface->pixels + p.getY()*sdlSurface->pitch + p.getX()) = format->mapToPixel(c);
	break;
      case 2:
        *((Uint16 *)sdlSurface->pixels + p.getY()*sdlSurface->pitch/2 + p.getX()) = format->mapToPixel(c);
	break;
      case 3:
        i = (Uint8 *)sdlSurface->pixels + p.getY()*sdlSurface->pitch + 3*p.getX();
	*((Uint32 *)i) &= 0xFF000000;
	*((Uint32 *)i) |= (pix & 0xFFFFFF);
	break;
      case 4:
        ((Uint32 *)sdlSurface->pixels)[p.getY()*sdlSurface->pitch/4 + p.getX()] = pix;
	break;
    }
  }
}
Color Surface::_getpixel(const Point &p) {
  Uint8 *i;
  Uint32 c = 0;

  switch (format->bpp()) {
    case 1:
      c = *((Uint8 *)sdlSurface->pixels + p.getY()*sdlSurface->pitch + p.getX());
      break;
    case 2:
      c = *((Uint16 *)sdlSurface->pixels + p.getY()*sdlSurface->pitch/2 + p.getX());
      break;
    case 3:
      i = (Uint8 *)sdlSurface->pixels + p.getY()*sdlSurface->pitch + 3*p.getX();
	
      c = *((Uint32 *)i) & 0xFFFFFF;
      break;
    case 4:
      c = *((Uint32 *)sdlSurface->pixels + p.getY()*sdlSurface->pitch/4 + p.getX());
      break;
  }
  
  return format->mapToColor(c);
}

void Surface::setAlpha(Uint8 val) {
  SDL_SetAlpha(sdlSurface, SDL_SRCALPHA, val);
}

void Surface::filledPolygon(const Uint8 pcnt, const Point * const points, Color &fill, Color &border) {
  // for first draw just the boder
  for (int i = 0; i < pcnt-1; i++) 
    line(points[i], points[i+1], border);
    
  line(points[pcnt-1], points[0], border);
}