/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



#ifndef SURFACE__H_
#define SURFACE__H_

#include "color.h"
#include "point.h"
#include "rect.h"
#include "pixelformat.h"
#include "ttfont.h"

#include <SDL/SDL.h>

#include <string>

class TTFont;

/**
 * encapsulates a surface to paint on
 */
 
class Surface {
  public:

    /**
     * creates a surface without any sdl_surface so you are not
     * allowed to draw onto this
     */
    Surface();
    
    /**
     * creates a surface with the given size and the given Pixelformat
     */
    Surface(const int w, const int h, const Pixelformat &f);
    
    /**
     * copies the surface (creates a surface witht the same dimensions
     * and copies the contents)
     */
    Surface(const Surface& surf);
    
    /**
     * creates a surface containing this sdl_surface
     */
    Surface(SDL_Surface *s);
    
    virtual ~Surface();
    
    /**
     * loads the surface from the stream assuming the picture in the stream
     * is juats as big as the surface
     */
    void loadImage(istream &in);
    
    /**
     * loads a sprite. that is an image with transparent pixels
     */
    void loadSprite(istream &in, Color key=Color(255));

    /**
     * returns true if there is no sdlsurface to paint on
     */
    bool empty() const { return sdlSurface == NULL; };
    
    /**
     * returns the width of the surface
     */
    virtual int width() const { return ((sdlSurface == NULL) ? 0 : sdlSurface->w); }
    
    /**
     * returns the height if the surface
     */
    virtual int height() const { return ((sdlSurface == NULL) ? 0 : sdlSurface->h); }

    /**
     * blits the whole surface onto the destination surface on position p
     */
    void blit(Surface &, const Point &p) const;
    
    /**
     * blits the part of the surface to the point on the destination
     * surface
     */
    void blit(Surface &, const Rect&, const Point&) const;

    /**
     * fills the whole surface with the given color
     */
    void fill(const Color&);
    
    /**
     * draws a filled bar 
     */
    void bar(const Rect&, const Color&);
    
    /**
     * puts a pixel
     */
    void putpixel(const Point&, const Color&);
    
    Color getpixel(const Point&);
    
    /**
     * draws a line
     */
    void line(const Point&, const Point&, const Color&);
    
    /**
     * draws an empty circle
     */
//    void circle(const Point &, const Uint16 r, const Color &);
    
    /**
     * draws an empty ellipse
     */
//    void ellipse(const Point &, const Uint16 rx, const Uint16 ry, const Color &);
    
    /**
     * draws a filled circle
     */
    void filledCircle(const Point &, const Uint16 r, const Color &);
    
    /**
     * draws a filled ellipse
     */
//    void filledEllipse(const Point &, const Uint16 rx, const Uint16 ry, const Color &);

    /**
     * draws a filles polygon
     */
     void filledPolygon(const Uint8 pcnt, const Point * const points, Color &fill, Color &border);
    
    /**
     * outputs the given text with the given color and the font.
     * cDir and hDir specify what position the given point specifies of this text
     * a value bigger than zero means the text is above or to the right or the point
     * a value of zero means the text is evenly distrebuted arount this point and a value
     * smaller than zero meand the text is below and to the left of this point
     */
    void text(const Point &, const string &text, const Color &, TTFont &, const Sint8 vDir = 1, const Sint8 hDir = 1);
    
    SDL_Surface *getSurface() const { return sdlSurface; }
    
    /**
     * allows to set the clipping area of this surface
     */
    void setClipRect(const Rect &r);
    void clearClipRect();
    
    /**
     * allows to set the alpha blending value for the whole surface
     */
    void setAlpha(Uint8 val);
 
  protected:
  
    /**
     * the surface all the output goes to
     */
    SDL_Surface* sdlSurface;
    
    /**
     * the pixelformat of the surface
     */
    Pixelformat *format;
    
    // coordinates of the point in the lower right corner
    Point maxCoord;
    
    /**
     * function to lock the surface always
     */
    bool _lock();
    void _unlock();
    bool _mustlock();
    void _putpixel(const Point &p, const Color &c);
    Color _getpixel(const Point &p);
};

#endif
